import { world, system } from "@minecraft/server";

function handleItemDurability(player, item) {
    if (player.getGameMode() !== 'survival' || !item.typeId.startsWith("oh:")) return;

    const durabilityComp = item.getComponent("durability");
    if (!durabilityComp) return;

    const unbreakingLevel = item.getComponent("minecraft:enchantable")?.getEnchantment("unbreaking")?.level || 0;
    const shouldDamage = unbreakingLevel === 0 || Math.random() < (1 / (unbreakingLevel + 1));

    if (!shouldDamage) return;

    durabilityComp.damage += 1;

    if (durabilityComp.damage >= durabilityComp.maxDurability) {
        player.getComponent("inventory").container.getSlot(player.selectedSlotIndex).setItem(null);
        player.playSound("random.break", { pitch: 1, volume: 1 });
    } else {
        player.getComponent("inventory").container.getSlot(player.selectedSlotIndex).setItem(item);
    }
}

function isValidPosition(dimension) {
    return dimension.id === 'minecraft:overworld' || dimension.id === 'minecraft:nether';
}

function getDirection(player) {
    const yaw = player.getRotation().y;
    const pitch = player.getRotation().x;
    const directions = ["north", "east", "south", "west", "up", "down"];

    if (pitch <= -45) return "up";
    if (pitch >= 45) return "down";
    return directions[Math.floor((yaw + 225) / 90) % 4];
}

function performBlockRemoval(player, x, y, z, direction) {
    const directions = {
        north: [
            { x: 1, y: 0, z: 0 },
            { x: 1, y: 1, z: 0 },
            { x: 0, y: 1, z: 0 },
            { x: -1, y: 1, z: 0 },
            { x: -1, y: 0, z: 0 },
            { x: 1, y: -1, z: 0 },
            { x: 0, y: -1, z: 0 },
            { x: -1, y: -1, z: 0 }
        ],
        east: [
            { x: 0, y: 0, z: 1 },
            { x: 0, y: 1, z: 1 },
            { x: 0, y: 1, z: 0 },
            { x: 0, y: 1, z: -1 },
            { x: 0, y: 0, z: -1 },
            { x: 0, y: -1, z: 1 },
            { x: 0, y: -1, z: 0 },
            { x: 0, y: -1, z: -1 }
        ],
        south: [
            { x: -1, y: 0, z: 0 },
            { x: -1, y: 1, z: 0 },
            { x: 0, y: 1, z: 0 },
            { x: 1, y: 1, z: 0 },
            { x: 1, y: 0, z: 0 },
            { x: -1, y: -1, z: 0 },
            { x: 0, y: -1, z: 0 },
            { x: 1, y: -1, z: 0 }
        ],
        west: [
            { x: 0, y: 0, z: -1 },
            { x: 0, y: 1, z: -1 },
            { x: 0, y: 1, z: 0 },
            { x: 0, y: 1, z: 1 },
            { x: 0, y: 0, z: 1 },
            { x: 0, y: -1, z: -1 },
            { x: 0, y: -1, z: 0 },
            { x: 0, y: -1, z: 1 }
        ],
        up: [
            { x: 0, y: 0, z: 0 },
            { x: 1, y: 0, z: 0 },
            { x: -1, y: 0, z: 0 },
            { x: 0, y: 0, z: 1 },
            { x: 1, y: 0, z: 1 },
            { x: -1, y: 0, z: 1 },
            { x: 0, y: 0, z: -1 },
            { x: 1, y: 0, z: -1 },
            { x: -1, y: 0, z: -1 }
        ],
        down: [
            { x: 0, y: 0, z: 0 },
            { x: 1, y: 0, z: 0 },
            { x: -1, y: 0, z: 0 },
            { x: 0, y: 0, z: 1 },
            { x: 1, y: 0, z: 1 },
            { x: -1, y: 0, z: 1 },
            { x: 0, y: 0, z: -1 },
            { x: 1, y: 0, z: -1 },
            { x: -1, y: 0, z: -1 }
        ]
    };

    if (directions[direction]) {
        directions[direction].forEach(({ x: dx, y: dy, z: dz }) => {
            player.runCommand(`execute unless block ${x + dx} ${y + dy} ${z + dz} bedrock unless block ${x + dx} ${y + dy} ${z + dz} light_block unless block ${x + dx} ${y + dy} ${z + dz} barrier unless block ${x + dx} ${y + dy} ${z + dz} deny unless block ${x + dx} ${y + dy} ${z + dz} allow unless block ${x + dx} ${y + dy} ${z + dz} command_block unless block ${x + dx} ${y + dy} ${z + dz} obsidian unless block ${x + dx} ${y + dy} ${z + dz} border_block unless block ${x + dx} ${y + dy} ${z + dz} crying_obsidian unless block ${x + dx} ${y + dy} ${z + dz} reinforced_deepslate run setblock ${x + dx} ${y + dy} ${z + dz} air destroy`);
        });
    }
}
world.afterEvents.playerBreakBlock.subscribe(event => {
    const { x, y, z } = event.block.location;
    const { player } = event;
    const dimension = player.dimension;
    const item = player.getComponent("inventory").container.getSlot(player.selectedSlotIndex)?.getItem();
    const direction = getDirection(player);

    if (item) {
        handleItemDurability(player, item);
        if (item.hasTag("dexten:on_use_hammer") && isValidPosition(dimension)) {
            performBlockRemoval(player, x, y, z, direction);
        }
    }
});
